-- OpenGL stuff
-- © Riccardo Di Meo 2015

local znear = 0.1
local zfar  = 300.0
local clearColor = {0.0, 0.0, 0.0, 0.0}
local requiredextensions = {"GL_OES_EGL_image_external"}
--local prerendervideo_width  = 4096; -- Optional: if not specified is set to the maximum size for the framebuffer (it will never be bigger than that, though) 
--local prerendervideo_height = 4096; -- Optional: if not specified is set to the maximum size for the framebuffer (it will never be bigger than that, though)
local prerendervideo_force_downsizing = true; -- Optional, false by default: downsize the texture size to the size of the movie
local texturedir = "textures";

local function generate_debug_objects(common_uniforms) 
   return {
      axes = {
         {
            type = "bind program",
            shader = {
               vert = "shaders/axes/axes.vert", 
               frag = "shaders/axes/axes.frag",
            }
         },
         { 
            type = "bind uniforms", 
            uniforms = {
               projection = common_uniforms.projection,
               modelview  = common_uniforms.modelview
            }
         },
         {
            type = "bind attribute",
            name = "pos",
            values = {
               0, 0, 0,
               0.3, 0, 0,
               0, 0, 0,
               0, 0.3, 0,
               0, 0, 0,
               0, 0, 0.3,
            },
            dim = 3
         },
         {
            type = "bind attribute",
            name = "vcolor",
            values = {
               1, 0, 0,
               1, 0, 0,
               0, 1, 0,
               0, 1, 0,
               0, 0, 1,
               0, 0, 1,
            },
            dim = 3
         },
         {
            type = "draw",     
            primitive = "GL_LINES",
            elements = {
               0, 1, 2, 3, 4, 5
            },
         },
         {
            type = "bind attribute",
            name = "pos"
         },
         {
            type = "bind attribute",
            name = "vcolor"
         }
      },         
      -- Don't remove: this dot shows an example on how 
      -- to place a dot a the center of the player's view
      center = {
         {
            type = "bind program",
            shader = {
               vert = "shaders/dots/dots.vert", 
               frag = "shaders/dots/dots.frag",
            }
         },
         { -- uses the current shader
            type = "bind uniforms", 
            uniforms = {
               projection = common_uniforms.projection,
               modelview  = common_uniforms.modelview,
               color = {
                  content = {1.0, 0.0, 0.0, 1.0},
                  vtype = "vec4"
               }
            }
         },
         {
            type = "bind attribute",
            name = "posandsize",
            values = {
               0.0, 0, 0, 10.0
            },
            dim = 4
         },
         {
            type = "disable",
            name = "GL_DEPTH_TEST"
         },
         {
            type = "draw",     
            primitive = "GL_POINTS",
            count  = 1,
         },
         {
            type = "enable",
            name = "GL_DEPTH_TEST"
         },
         {  
            type = "bind attribute",
            name = "posandsize"
         },
      }  
   }
end

local M = {
   texturedir = texturedir,
   prerendervideo_force_downsizing = prerendervideo_force_downsizing,
   prerendervideo_height = prerendervideo_height,
   prerendervideo_width = prerendervideo_width,
   prerendervideo = prerendervideo,
   requiredextensions = requiredextensions,
   zfar = zfar,
   znear = znear,
   clearColor = clearColor,
   generate_debug_objects = generate_debug_objects
}
return M
