-- Collection of scene options
-- © Riccardo Di Meo 2015

local mymath = require"mymath"

local options = {}

-- Room geometry
options.room_depth  = 25.0
options.room_width  = 20.0
options.room_height = 14.0
options.floor_level = 0.0

-- Phong model parameters
-- I should multiply 1.42 *to the incoming light*, not the phong parameters
options.ceil_ads        = {0.03 * 1.42, 0.23 * 1.42, 0.0 * 1.42}
options.rug_ads         = {0.15 * 1.42, 0.5  * 1.42, 1.0  * 1.42}
options.curtains_ads    = {0.10 * 1.42, 0.12  * 1.42, 1.0  * 1.42}
options.cardboard_ads   = {0.15 * 20.00 * 0.6, 0.9  * 20.00 * 0.6, 1.0  * 20.00 * 0.0}
-- no back wall: it renders the same as the front one
options.front_ads       = {0.05 * 1.42, 0.0  * 1.42, 0.0  * 1.42}

-- Light source parameters (static lighting)
options.light_intensity = 1.0
options.light_near      = 9.0
options.light_far       = 12.0

-- Used to compute the positions of the keys
options.ui = {
   distance = 3,
   button_side = 0.18,
   buttons_radius = 0.6,
   -- used only for panoramic. About the get extinct
   angle    = mymath.grad2rad(60),
   spacing  = 0.7,
}

return options
