-- Functional stuff (map, filter, ecc)
-- Created by Riccardo Di Meo
-- © Riccardo Di Meo 2015

methods = {}

methods.identity = function(x) return x end

methods.shallow_copy = function(t)
   return methods.map_obj(t, methods.identity)
end

methods.append_array = function(first, second)
   local result = methods.shallow_copy(first)
   for _, j in ipairs(second) do
      result[#result + 1] = j
   end     
   return result
end

methods.map_obj = function(t, f)
   local result = {}
   for i, j in pairs(t) do
      result[i] = f(j, i)
   end
   return result
end

methods.map = function(array, f)
   local result = {}
   for i=1,#array do 
      result[i] = f(array[i], i)
   end
   return result
end

methods.explode_on_nil = function(value)
   if value == nil then
      error("Unexpected nil value!")
   else
      return value
   end
end

methods.merge_maps = function(first, second)
   local result = {}
   for i, j in pairs(first) do
      result[i] = j
   end
   for i, j in pairs(second) do
      result[i] = j
   end
   return result
end

methods.filter = function(array, predicate)
   local result = {}
   for i, j in ipairs(array) do 
      if predicate(j, i) then
         result[#result + 1] = j
      end
   end
   return result
end

return methods
