local common_uniforms = require "scene/common_uniforms"
local opengl_common = require "scene/opengl_common"
local options_consts = require "scene/options_consts"
local scene_controls_opengl = require "scene/scene_controls_opengl"
local functional = require "scene/functional"
local vr_config = require "scene/vr_config"
local consts = require "scene/scene_constants"

local function create_opengl_objects(parameters)
    local direct_rendering = parameters.content_type ~= "image"

    local stereo_type = parameters.stereo_type

    -- Find out which datafile to use (either sphere or semi-sphere
    local datafile
    if parameters.ratio == (360.0 / 180) then
        datafile = "scene/panoramic/" .. consts.ARG_SCREEN_360
    else
        datafile = "scene/panoramic/" .. consts.ARG_SCREEN_180
    end

    local screen_attributes = require(datafile)

    local opengl_objects = {}

    local tmp_screen = {
        uniforms = {
            projection = common_uniforms.projection,
            model = common_uniforms.model,
            view = common_uniforms.view,
            eye = common_uniforms.eye,
            id = common_uniforms.id,
            brightness = common_uniforms.brightness,
            contrast = common_uniforms.contrast,
            feedback_rendering = common_uniforms.feedback_rendering,
        }
    }
    -- different shaders for different situations
    if direct_rendering then
        tmp_screen.uniforms.video = common_uniforms.video
        tmp_screen.uniforms.video_transform = common_uniforms.video_transform
    else
        tmp_screen.uniforms.prevideo = common_uniforms.video
    end


    -- those two are used only when "anaglyph red/cyan" is selected
    if stereo_type == consts.ARG_STEREO_TYPE_ANAGLYPH_RED_CYAN then
        tmp_screen.uniforms.swizzle_right = { content = opengl_common.anaglyph_conversion(consts.COLOR_RED), vtype = "mat3" }
        tmp_screen.uniforms.swizzle_left = { content = opengl_common.anaglyph_conversion(consts.COLOR_CYAN), vtype = "mat3" }
    end

    opengl_objects.screen = {
        {
            type = "bind program",
            shader = {
                vert = opengl_common.get_screen_vertex_shader(stereo_type, parameters.ratio, direct_rendering),
                frag = opengl_common.get_screen_fragment_shader(stereo_type, parameters.ratio, direct_rendering),
            }
        },
        {
            -- uses the current shader
            type = "bind uniforms",
            uniforms = tmp_screen.uniforms
        },
        {
            type = "bind attribute",
            name = "pos",
            values = screen_attributes.pos.values,
            dim = screen_attributes.pos.dim
        },
        {
            type = "bind attribute",
            name = "uv",
            values = screen_attributes.uv.values,
            dim = screen_attributes.uv.dim
        },
        {
            type = "draw",
            count = #screen_attributes.pos.values / screen_attributes.pos.dim,
            primitive = "GL_TRIANGLES",
        },
        {
            type = "bind attribute",
            name = "uv"
        },
        {
            type = "bind attribute",
            name = "pos"
        }
    }
    tmp_screen = nil

    screen_attributes = nil -- not strictly required, if not for garbage collection

    opengl_objects = functional.merge_maps(opengl_objects,
        scene_controls_opengl.generate_control_objects(nil, false, true, parameters.ratio, parameters.stereo_type))

    opengl_objects.progress = opengl_common.progress_bar_model(common_uniforms)
    opengl_objects.pointer = scene_controls_opengl.return_pointer(0.2,
        vr_config.get(options_consts.POINTER_OPACITY) / 100.0)

    return opengl_objects
end

return {
    create_opengl_objects = create_opengl_objects
}
