-- Modulo per le operazioni sui quaternioni. Solo quello che mi interessa.
--
-- Testato in "math_testing.lua" in scripting
-- Created by Riccardo Di Meo
-- © Riccardo Di Meo 2015

local mm = require "mymath"

-- the real-world limits of equality
local EPS=0.0000001

local function length(q) 
   return math.sqrt(q[1] * q[1] + q[2] * q[2] + q[3] * q[3] + q[4] * q[4])
end

local function normalize(q)
   local l = length(q)
   return { q[1] / l, q[2] / l, q[3] / l, q[4] / l}
end

local function from_axis_angle(axis, angle)
   local x, y, z = table.unpack(mm.normalize_3D(axis))
   local sin = math.sin(angle * 0.5)
   return {math.cos(angle * 0.5), x * sin, y * sin, z * sin}
end

local function qtprint(q) 
   print("Q(" .. q[1] .. ", " .. q[2] .. ", " .. q[3] .. ", " .. q[4] .. ")")
end

local function to_axis_angle(q) 
   local hangle = math.acos(q[1])
   local s = math.sin(hangle)
   if math.abs(s) < EPS then
      return {1, 0, 0}, 0
   else
      return mm.normalize_3D({ q[2] / s, q[3] / s, q[4] / s }), hangle * 2
   end
end

local function print_3D_matrix(m) 
   print("| [" .. m[1][1] .. "] [" .. m[2][1] .. "] [" .. m[3][1] .. "]")
   print("| [" .. m[1][2] .. "] [" .. m[2][2] .. "] [" .. m[3][2] .. "]")
   print("| [" .. m[1][3] .. "] [" .. m[2][3] .. "] [" .. m[3][3] .. "]")
end

-- kindly lifted from http://www.euclideanspace.com/maths/geometry/rotations/conversions/quaternionToMatrix/index.htm
local function quaternion_to_matrix(q) 
    local sqw = q[1] * q[1]
    local sqx = q[2] * q[2]
    local sqy = q[3] * q[3]
    local sqz = q[4] * q[4]

    -- invs (inverse square length) is only required if quaternion is not already normalised
    local invs = 1 / (sqx + sqy + sqz + sqw)
    local m = {{0, 0, 0}, {0, 0, 0}, {0, 0, 0}}
    m[1][1] = (  sqx - sqy - sqz + sqw ) * invs  -- since sqw + sqx + sqy + sqz =1/invs*invs
    m[2][2] = ( -sqx + sqy - sqz + sqw ) * invs 
    m[3][3] = ( -sqx - sqy + sqz + sqw ) * invs 
    
    local tmp1 = q[2] * q[3]
    local tmp2 = q[4] * q[1]
    m[2][1] = 2.0 * ( tmp1 + tmp2 ) * invs 
    m[1][2] = 2.0 * ( tmp1 - tmp2 ) * invs 
    
    tmp1 = q[2] * q[4]
    tmp2 = q[3] * q[1]
    m[3][1]  = 2.0 * ( tmp1 - tmp2 ) * invs 
    m[1][3]  = 2.0 * ( tmp1 + tmp2 ) * invs 
    tmp1 = q[3] * q[4]
    tmp2 = q[2] * q[1]
    m[3][2]  = 2.0 * ( tmp1 + tmp2 ) * invs 
    m[2][3]  = 2.0 * ( tmp1 - tmp2 ) * invs       
    return m
end

local function axis_angle_to_matrix(axis, angle)
   return quaternion_to_matrix(from_axis_angle(axis, angle))
end

-- freely adapted from the code provided here:
--   http://www.euclideanspace.com/maths/geometry/rotations/conversions/matrixToQuaternion/
--local function matrix_to_quaternion(a)
--   local trace = a[1][1] + a[2][2] + a[3][3]
--   local q = {} -- the output
--   if trace > 0 then
--      print "PATH 1"
--      local s = 0.5 / math.sqrt(trace + 1.0)
--      q[1] = 0.25 / s
--      q[2] = ( a[3][2] - a[2][3] ) * s
--      q[3] = ( a[1][3] - a[3][1] ) * s
--      q[4] = ( a[2][1] - a[1][2] ) * s
--   else      
--      if a[1][1] > a[2][2] and a[1][1] > a[3][3] then
--	 local s = 2.0 * math.sqrt( 1.0 + a[1][1] - a[2][2] - a[3][3])
--         print("Path 2", s)
--	 q[1] = (a[3][2] - a[2][3] ) / s
--	 q[2] = 0.25 * s
--	 q[3] = (a[1][2] + a[2][1] ) / s
--	 q[4] = (a[1][3] + a[3][1] ) / s
--      elseif (a[2][2] > a[3][3]) then
--	 local s = 2.0 * math.sqrt( 1.0 + a[2][2] - a[1][1] - a[3][3])
--         print("Path 3", s)
--	 q[1] = (a[1][3] - a[3][1] ) / s
--	 q[2] = (a[1][2] + a[2][1] ) / s
--	 q[3] = 0.25 * s
--	 q[4] = (a[2][3] + a[3][2] ) / s
--      else 
--	 local s = 2.0 * math.sqrt( 1.0 + a[3][3] - a[1][1] - a[2][2] )
--         print("Path 4", s)
--	 q[1] = (a[2][1] - a[1][2] ) / s
--	 q[2] = (a[1][3] + a[3][1] ) / s
--	 q[3] = (a[2][3] + a[3][2] ) / s
--	 q[4] = 0.25 * s
--      end	            
--   end
--   return normalize(q)
--end

local function matrix_to_quaternion(m)
   local Q = {}
   local sqrt = math.sqrt
   local max = math.max
   Q.w = sqrt( max( 0, 1 + m[1][1] + m[2][2] + m[3][3] ) ) / 2;
   Q.x = sqrt( max( 0, 1 + m[1][1] - m[2][2] - m[3][3] ) ) / 2;
   Q.y = sqrt( max( 0, 1 - m[1][1] + m[2][2] - m[3][3] ) ) / 2;
   Q.z = sqrt( max( 0, 1 - m[1][1] - m[2][2] + m[3][3] ) ) / 2;

   local function _copysign(x, y)
      if y > 0 then
         return math.abs(x)
      else
         return -math.abs(x)
      end
   end

   return {
      Q.w, 
      _copysign( Q.x, m[3][2] - m[2][3] ), 
      _copysign( Q.y, m[1][3] - m[3][1] ), 
      _copysign( Q.z, m[2][1] - m[1][2] )}
end


local function matrix_to_axis_angle(m) 
   local q = matrix_to_quaternion(m)
   return to_axis_angle(q)
end

local front_up_to_quaternion = function(front, up) 
   front = mm.normalize_3D(front)
   up = mm.normalize_3D(up)
   local right = mm.vec_product(up, front)
   return matrix_to_quaternion({right, up, front})
end

return {
   print  = qtprint,
   length = length,
   from_axis_angle = from_axis_angle,
   to_axis_angle = to_axis_angle,
   normalize = normalize,
   axis_angle_to_matrix = axis_angle_to_matrix,
   print_3D_matrix = print_3D_matrix,
   matrix_to_quaternion = matrix_to_quaternion,
   matrix_to_axis_angle = matrix_to_axis_angle,
   front_up_to_quaternion = front_up_to_quaternion,
   quaternion_to_matrix = quaternion_to_matrix,
}
