-- 3D Scene details
-- Created by Riccardo Di Meo
-- © Riccardo Di Meo 2015
local options = require "options"
local options_consts = require "scene/options_consts"
local vr_config = require "scene/vr_config"

local THEATER_LIKE_ENV = {
    [options_consts.VOID_ENV] = true,
    [options_consts.THEATER_ENV] = true,
    [options_consts.CARDBOARD_THEATER_ENV] = true,
    [options_consts.CARDBOARD_CARDBOARD_THEATER_ENV] = true,
}

local function create_and_clean(f_name, module_name, parameters)
    -- yep: forget about the environment
    local mod = require(module_name)
    local result = mod[f_name](parameters)
    mod = nil
    collectgarbage("collect")
    return result
end

-- Public function (the only one, as a matter of fact…
local create_scene_with_parameters = function(parameters)
    local environment = vr_config.get(options_consts.THEATER_TYPE)

    --   print " ===== PARAMETERS"
    --   for i,j in pairs(parameters) do
    --      print("  +", i, "->", j)
    --   end

    -- is the movie a panoramic one?
    if parameters.ratio == (360.0 / 180) or parameters.ratio == (180.0 / 180.0) then
        return create_and_clean("create", "scene/panoramic/scene", parameters)
    else
        -- no: select the correct environment
        if THEATER_LIKE_ENV[environment] then
            return create_and_clean("create", "scene/theater/scene", parameters)
        end

        if environment == options_consts.CUBEMAP_THEATER then
           return create_and_clean("create", "scene/cubemap/scene", parameters)
        end
    end
    error "No suitable scene found"
end

local M = {}
M.visibility_query_period = visibility_query_period
M.create_scene_with_parameters = create_scene_with_parameters
return M
