local opengl       = require "opengl"
local options      = require "options"
local consts       = require "scene/scene_constants"
local options_consts = require "scene/options_consts"
local scene_debug  = require "scene/scene_debug"
local vr_config    = require "scene/vr_config"

local function get_scene_object_named(objects_array, name) 
   for index, object in ipairs(objects_array) do
      if object.name == name then
         return index, object
      end
   end
   return nil, nil
end

local function light_parameters(screen_params)
   return {
      ceil   = { 
         range = { content = {(-screen_params.width * 0.5), screen_params.width * 0.5}, vtype = "vec2"}, 
         position = { content = {0.0, screen_params.lift + 0.5 * screen_params.height, -options.room_depth * 0.5 + 0.01, 1.0}, vtype = "vec4"}
      },
      right   = { 
         range    = { content = {screen_params.lift - screen_params.height * 0.5, screen_params.lift + screen_params.height * 0.5}, vtype = "vec2" }, 
         position = { content = {screen_params.width * 0.5, 0, -options.room_depth * 0.5 + 0.01, 1.0}, vtype = "vec4" }
      },
      left  = { 
         range    = { content = {screen_params.lift - screen_params.height * 0.5, screen_params.lift + screen_params.height * 0.5}, vtype = "vec2" },
         position = { content = {-screen_params.width * 0.5, 0, -options.room_depth * 0.5 + 0.01, 1.0}, vtype = "vec4" }
      },
      floor = { 
         range    = { content = {(-screen_params.width * 0.5), screen_params.width * 0.5}, vtype = "vec2" }, 
         position = { content = {0.0, screen_params.lift - 0.5 * screen_params.height, -options.room_depth * 0.5 + 0.01, 1.0}, vtype = "vec4" }
      }
   }
end

local function correct_light_base(uniform)

   local light = 3 * vr_config.get(options_consts.LIGHT_MULTIPLIER) / (100.0 * 2) - 0.2

   if scene_debug.scene.intensify_light then
      light = light * scene_debug.scene.intensify_light_factor
   end

   uniform.content[1] = uniform.content[1] * light
   uniform.content[2] = uniform.content[2] * light
   uniform.content[3] = uniform.content[3] * light   
   return uniform
end

local function correct_light_cardboard(uniform)

   local light = vr_config.get(options_consts.LIGHT_MULTIPLIER) / (100.0 * 2) + (3.0 / 10)

   if scene_debug.scene.intensify_light then
      light = light * scene_debug.scene.intensify_light_factor
   end

   uniform.content[1] = uniform.content[1] * light
   uniform.content[2] = uniform.content[2] * light
   uniform.content[3] = uniform.content[3] * light   
   return uniform
end
local return_cardboard_theater_environment_uniforms = function(screen_params)
   local lparams = light_parameters(screen_params)

   local room_ratio =  options.room_depth / options.room_width

   uniforms = {}
   uniforms.rightwall_lightpos = lparams.right.position
   uniforms.rightwall_lightrange = lparams.right.range
   uniforms.rightwall_diffuse  = { content = "TEX:1:" .. opengl.texturedir .. "/curtains/diffuse.png", vtype = "int"}
   uniforms.rightwall_normal   = { content = "TEX:2:" .. opengl.texturedir .. "/curtains/normal.png", vtype = "int"}
   uniforms.rightwall_specular = { content = "TEX:3:" .. opengl.texturedir .. "/curtains/specular.png", vtype = "int"}
   uniforms.rightwall_uvscale  = { content = {1.3, 1.0}, vtype = "vec2"}
   uniforms.rightwall_uvoffset = { content = {0.0, 0.0}, vtype = "vec2"}
   uniforms.rightwall_phongweights = correct_light_base({ content = options.curtains_ads, vtype = "vec3"})

   uniforms.leftwall_lightpos = lparams.left.position
   uniforms.leftwall_lightrange = lparams.left.range
   uniforms.leftwall_diffuse  = { content = "TEX:1:" .. opengl.texturedir .. "/curtains/diffuse.png", vtype = "int"}
   uniforms.leftwall_normal   = { content = "TEX:2:" .. opengl.texturedir .. "/curtains/normal.png", vtype = "int"}
   uniforms.leftwall_specular = { content = "TEX:3:" .. opengl.texturedir .. "/curtains/specular.png", vtype = "int"}
   uniforms.leftwall_uvscale = { content = {1.3, 1.0}, vtype = "vec2"}
   uniforms.leftwall_uvoffset = { content = {0.5, 0.0}, vtype = "vec2"}
   uniforms.leftwall_phongweights = correct_light_base({ content = options.curtains_ads, vtype = "vec3"})

   uniforms.frontwall_diffuse  = { content = "TEX:1:" .. opengl.texturedir .. "/concrete/diffuse.png", vtype = "int"}
   uniforms.frontwall_normal   = { content = "TEX:2:" .. opengl.texturedir .. "/concrete/normal.png", vtype = "int"}
   uniforms.frontwall_specular = { content = "TEX:3:" .. opengl.texturedir .. "/concrete/specular.png", vtype = "int"}
   uniforms.frontwall_uvscale  = { content = {4.0, 4.0 * options.room_height / options.room_width}, vtype = "vec2"}
   uniforms.frontwall_uvoffset = { content = {0.0, 0.0}, vtype = "vec2"}
   uniforms.frontwall_phongweights = correct_light_base({ content = options.front_ads, vtype = "vec3"})
   
   uniforms.floor_lightrange   = lparams.floor.range
   uniforms.floor_lightpos     = lparams.floor.position
   uniforms.floor_diffuse      = { content = "TEX:1:" .. opengl.texturedir .. "/rug/diffuse.png", vtype = "int"}
   uniforms.floor_normal       = { content = "TEX:2:" .. opengl.texturedir .. "/rug/normal.png", vtype = "int"}
   uniforms.floor_specular     = { content = "TEX:3:" .. opengl.texturedir .. "/rug/specular.png", vtype = "int"}
   uniforms.floor_uvscale      = { content = {2.5, 2.5 * room_ratio}, vtype = "vec2"}
   uniforms.floor_uvoffset     = { content = {0.0, 0.0}, vtype = "vec2"}
   uniforms.floor_phongweights = correct_light_base({ content = options.rug_ads, vtype = "vec3"})

   
   uniforms.ceil_lightrange = lparams.ceil.range
   uniforms.ceil_lightpos   = lparams.ceil.position
   uniforms.ceil_diffuse  = { content = "TEX:1:" .. opengl.texturedir .. "/tiles/diffuse.png", vtype = "int"}
   uniforms.ceil_normal   = { content = "TEX:2:" .. opengl.texturedir .. "/tiles/normal.png", vtype = "int"}
   uniforms.ceil_specular = { content = "TEX:3:" .. opengl.texturedir .. "/tiles/specular.png", vtype = "int"}
   uniforms.ceil_uvscale = { content = {1.2, 1.2 * room_ratio}, vtype = "vec2"}
   uniforms.ceil_uvoffset = { content = {0.0, 0.0}, vtype = "vec2"}
   uniforms.ceil_phongweights = correct_light_base({ content = options.ceil_ads, vtype = "vec3"})

   uniforms.common_ambientcolor = { content = {0.6, 0.8, 0.9}, vtype = "vec3"}
   uniforms.common_lightparams = { content = {options.light_intensity, options.light_near, options.light_far}, vtype = "vec3"} -- intensity, and start/end parameters for the falloffGame function
   uniforms.common_lightanimpar = { content = {0.0, 0.0, 2000}, vtype = "vec3"} -- this one is used for debugging only...

   return uniforms
end

local return_cardboard_cardboard_theater_environment_uniforms = function(screen_params)
   local lparams = light_parameters(screen_params)

   uniforms = {}
   uniforms.rightwall_lightrange = lparams.right.range
   uniforms.rightwall_lightpos   = lparams.right.position
   uniforms.rightwall_diffuse  = { content = "TEX:1:" .. opengl.texturedir .. "/cardboard/1/diffuse.png", vtype = "int"}
   uniforms.rightwall_normal   = { content = "TEX:2:" .. opengl.texturedir .. "/cardboard/1/normal.png", vtype = "int"}
   uniforms.rightwall_specular = { content = "TEX:3:" .. opengl.texturedir .. "/cardboard/1/specular.png", vtype = "int"}
   uniforms.rightwall_uvscale  = { content = {1.0, 1.0}, vtype = "vec2"}
   uniforms.rightwall_uvoffset = { content = {0.0, 0.0}, vtype = "vec2"}
   uniforms.rightwall_phongweights = correct_light_cardboard({ content = options.cardboard_ads, vtype = "vec3"})
   
   uniforms.leftwall_lightrange = lparams.left.range
   uniforms.leftwall_lightpos   = lparams.left.position
   uniforms.leftwall_diffuse  = { content = "TEX:1:" .. opengl.texturedir .. "/cardboard/2/diffuse.png", vtype = "int"}
   uniforms.leftwall_normal   = { content = "TEX:2:" .. opengl.texturedir .. "/cardboard/2/normal.png", vtype = "int"}
   uniforms.leftwall_specular = { content = "TEX:3:" .. opengl.texturedir .. "/cardboard/2/specular.png", vtype = "int"}
   uniforms.leftwall_uvscale  = { content = {1.0, 1.0}, vtype = "vec2"}
   uniforms.leftwall_uvoffset = { content = {0.0, 0.0}, vtype = "vec2"}
   uniforms.leftwall_phongweights = correct_light_cardboard({ content = options.cardboard_ads, vtype = "vec3"})
   
   uniforms.frontwall_diffuse  = { content = "TEX:1:" .. opengl.texturedir .. "/cardboard/3/diffuse.png", vtype = "int"}
   uniforms.frontwall_normal   = { content = "TEX:2:" .. opengl.texturedir .. "/cardboard/3/normal.png", vtype = "int"}
   uniforms.frontwall_specular = { content = "TEX:3:" .. opengl.texturedir .. "/cardboard/3/specular.png", vtype = "int"}
   uniforms.frontwall_uvscale  = { content = {1.0, 1.0}, vtype = "vec2"}
   uniforms.frontwall_uvoffset = { content = {0.0, 0.0}, vtype = "vec2"}
   uniforms.frontwall_phongweights = correct_light_cardboard({ content = options.cardboard_ads, vtype = "vec3"})
   uniforms.frontwall_phongweights.content[2] = 0.0
   uniforms.frontwall_phongweights.content[3] = 0.0

   uniforms.floor_lightrange = lparams.floor.range
   uniforms.floor_lightpos   = lparams.floor.position
   uniforms.floor_diffuse  = { content = "TEX:1:" .. opengl.texturedir .. "/cardboard/2/diffuse.png", vtype = "int"}
   uniforms.floor_normal   = { content = "TEX:2:" .. opengl.texturedir .. "/cardboard/2/normal.png", vtype = "int"}
   uniforms.floor_specular = { content = "TEX:3:" .. opengl.texturedir .. "/cardboard/2/specular.png", vtype = "int"}
   uniforms.floor_uvscale  = { content = {1.0, 1.0}, vtype = "vec2"}
   uniforms.floor_uvoffset = { content = {0.0, 0.0}, vtype = "vec2"}
   uniforms.floor_phongweights = correct_light_cardboard({ content = options.cardboard_ads, vtype = "vec3"})

   uniforms.ceil_lightrange = lparams.ceil.range
   uniforms.ceil_lightpos   = lparams.ceil.position
   uniforms.ceil_diffuse  = { content = "TEX:1:" .. opengl.texturedir .. "/cardboard/2/diffuse.png", vtype = "int"}
   uniforms.ceil_normal   = { content = "TEX:2:" .. opengl.texturedir .. "/cardboard/2/normal.png", vtype = "int"}
   uniforms.ceil_specular = { content = "TEX:3:" .. opengl.texturedir .. "/cardboard/2/specular.png", vtype = "int"}
   uniforms.ceil_uvscale  = { content = {1.0, 1.0}, vtype = "vec2"}
   uniforms.ceil_uvoffset = { content = {0.0, 0.0}, vtype = "vec2"}
   uniforms.ceil_phongweights = correct_light_cardboard({ content = options.cardboard_ads, vtype = "vec3"})

   uniforms.common_lightpos = { content = {0.0, screen_params.lift, -options.room_depth * 0.5 + 0.01, 1.0}, vtype = "vec4"} -- this is in world coordinates!!!
   uniforms.common_ambientcolor = { content = {0.5, 0.7, 0.8}, vtype = "vec3"}
   uniforms.common_lightparams = { content = {options.light_intensity, options.light_near, options.light_far}, vtype = "vec3"} -- intensity, and start/end parameters for the falloffGame function
   uniforms.common_lightanimpar = { content = {0.0, 0.0, 2000}, vtype = "vec3"} -- this one is used for debugging only...

   return uniforms
end

return {
   return_cardboard_theater_environment_uniforms = return_cardboard_theater_environment_uniforms,
   return_cardboard_cardboard_theater_environment_uniforms = return_cardboard_cardboard_theater_environment_uniforms,
   get_scene_object_named = get_scene_object_named,
}
