-- Creation and positioning of the subtitles in the scene
-- Created by Riccardo Di Meo
-- © Riccardo Di Meo 2016
local mymath = require "mymath"

local compute_position = function(distance, coordinates, camera_neg) 
   local camera_pos = mymath.neg_3D(camera_neg.position)
   
   local tl = coordinates[1]
   local br = coordinates[2]

   bl = mymath.copy_3D(tl)
   bl[2] = br[2]

   -- center bottom of the screen
   local cb = mymath.smul_3D(mymath.add_3D(br, bl), 0.5)

   -- Vector from camera to bottom center of screen
   local camera_to_cb = mymath.sub_3D(cb, camera_pos)

   -- camera -> bottom center of screen, proportional to distance,
   -- applied to camera
   local position = mymath.add_3D(mymath.smul_3D(camera_to_cb, distance), camera_pos)
   
   -- print("=== NEW ALGORITHM RESULT   : ", mymath.to_string_3D(position))
   -- print("... CAMERA POSITION [ORIG] : ", mymath.to_string_3D(camera_neg.position))
   -- print()
   -- print("... SCREEN TOP LEFT        : ", mymath.to_string_3D(coordinates[1]))
   -- print("... SCREEN BOTTOM RIGHT    : ", mymath.to_string_3D(coordinates[2]))
   return position
end

local compute_size = function(distance, scale, coordinates)
   local diff = mymath.smul_3D(mymath.sub_3D(coordinates[1], coordinates[2]), distance * scale)
   return math.abs(diff[1]), math.abs(diff[2])
end

local M = {
   compute_position = compute_position,
   compute_size     = compute_size,
}
return M
