#version 300 es
precision mediump float;

in vec2 fuv;
in vec4 uV;
in vec4 uLp;
out vec4 frag_color;

uniform highp mat4 modelview;
uniform vec3 lightparams;
uniform sampler2D diffuse;
uniform sampler2D normal;
uniform sampler2D specular;
//uniform sampler2D bump;

uniform vec3 phongweights;
uniform vec3 ambientcolor;

float falloffGL(float r, float sc, float sl, float sq) {
  return 1.0 / (sc + sl * r + sq * r * r);
}

// Src: Akenine-Moller, 219
float falloffGame(float r, float rstart, float rend) {
  return clamp((rend - r) / (rend - rstart), 0.0, 1.0);
}

vec3 simpleDiffuse(vec2 uv, vec3 light, vec3 normal) {
	float diffcoeff = max(dot(light, normal), 0.0);
	vec3  diffusecolor = texture(diffuse, uv).rgb;

	return diffusecolor * diffcoeff;
}

lowp vec3 simpleSpecular(vec2 uv, vec3 light, vec3 normal, vec3 eye) {
	highp vec3 reflection = reflect(-light, normal);
	highp float speccoeff = pow(max(dot(reflection, eye), 0.0), 64.0);

	highp vec3 specularcolor = vec3(1.0, 1.0, 1.0);

	return specularcolor * speccoeff * texture(specular, uv).x;
}

void main() {

  vec3 N = normalize(mat3(modelview) * (vec3(0.5) - texture(normal, fuv).rgb));
  //N = -(mat3(modelview) * vec3(0.0, 0.0, 1.0)); // Use this to debug the lighting
  vec3 V = -normalize(uV.xyz / uV.w); 
  vec3 tmp = uLp.xyz / uLp.w - uV.xyz / uV.w;
  float intensity = lightparams[0] * falloffGame(length(tmp), lightparams[1], lightparams[2]);
  vec3 L = normalize(tmp);

  vec3 ambientterm  = phongweights.x * (ambientcolor * texture(diffuse, fuv).rgb);
  vec3 diffuseterm  = phongweights.y * intensity * simpleDiffuse(fuv, L, N);
  vec3 specularterm = phongweights.z * intensity * simpleSpecular(fuv, L, N, V);

  frag_color.rgb = (ambientterm + diffuseterm + specularterm);
  frag_color.a = 1.0;

  // Debug statements (quite useful sometimes)
  //frag_color.rgb = pow(max(dot(reflect(-L, N), V), 0.0), 64.0) * vec3(1.0, 1.0, 1.0);
  //frag_color.rgb = L * L;
}
